//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "RVDocParams.hpp"
#include "RVSideNote.hpp"
#include "RVGrHandler.hpp"

#include "MainFrm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "RVScroll"
#pragma link "SclRView"
#pragma resource "*.dfm"
TfrmMain *frmMain;


//---------------------------------------------------------------------------
// Setting A4 page size and margins
void SetDocParameters(TSRichViewEdit* SRV)
{
  TSRVPageProperty * PP = SRV->PageProperty;
  SRV->UnitsProgram = rvuMillimeters;
  PP->PageWidth = 210;
  PP->PageHeight = 297;
  PP->LeftMargin = 30;
  PP->RightMargin = 15;
  PP->TopMargin = 20;
  PP->BottomMargin = 20;
  PP->HeaderY = 10;
  PP->FooterY = 10;
}
//---------------------------------------------------------------------------
void GenerateSampleDoc(TCustomRichView* RV)
{
  const TRVUnicodeString Text =
	"Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
	"Nullam enim turpis, gravida quis justo et, congue aliquam nisi. "
	"Curabitur placerat ex elit. Cras aliquam quam in mi accumsan, "
	"at pellentesque lectus finibus. Donec nisl nulla, scelerisque eget "
	"elit ac, rhoncus mattis eros. Maecenas gravida tincidunt diam, "
	"nec laoreet nulla gravida nec. Class aptent taciti sociosqu ad "
	"litora torquent per conubia nostra, per inceptos himenaeos. "
	"Nulla a metus eleifend nibh aliquam luctus sed vel quam. "
	"Vestibulum lobortis blandit iaculis.";

  RV->Clear();
  for (int i = 0; i <= 500; i++)
	RV->AddNL(Text, 0, 0);
}
//---------------------------------------------------------------------------
int GetCenteredParaStyle(TRVStyle* ARVStyle)
{
  TParaInfo* ParaStyle = new TParaInfo(NULL);
  ParaStyle->Assign(ARVStyle->ParaStyles->Items[0]);
  ParaStyle->Alignment = rvaCenter;
  int r = ARVStyle->FindParaStyle(ParaStyle);
  ParaStyle->Free();
  return r;
}
//---------------------------------------------------------------------------
void InsertTextBox(TSRichViewEdit* SRV)
{
  const double MarginsCoeff = 0.9;
  const double ImageCoeff = 0.9;
  TCustomRVData* Header = SRV->SubDocuments->Items[srvhftNormalHeader];
  TRVStyle* Style = Header->GetRVStyle();
  TSRVPageProperty * PP = SRV->PageProperty;
  TRVTextBoxItemInfo* TextBox = new TRVTextBoxItemInfo(Header);
  // position
  TextBox->BoxPosition->HorizontalPositionKind = rvfpkAbsPosition;
  TextBox->BoxPosition->HorizontalAnchor = rvhanPage;
  TextBox->BoxPosition->HorizontalOffset =
  Style->RVUnitsToUnits(PP->LeftMargin * MarginsCoeff, SRV->UnitsProgram);
  TextBox->BoxPosition->VerticalPositionKind = rvfpkAbsPosition;
  TextBox->BoxPosition->VerticalAnchor = rvvanPage;
  TextBox->BoxPosition->VerticalOffset =
  Style->RVUnitsToUnits(PP->TopMargin  * MarginsCoeff, SRV->UnitsProgram);
  TextBox->BoxPosition->PositionInText = rvpitBelowText;
  // size
  TextBox->BoxProperties->WidthType = rvbwtAbsolute;
  TextBox->BoxProperties->Width =
	Style->RVUnitsToUnits(
	  PP->PageWidth - (PP->LeftMargin + PP->RightMargin)  * MarginsCoeff, SRV->UnitsProgram);
  TextBox->BoxProperties->HeightType = rvbhtAbsolute;
  TextBox->BoxProperties->Height =
	Style->RVUnitsToUnits(
	  PP->PageHeight - (PP->TopMargin + PP->BottomMargin)  * MarginsCoeff, SRV->UnitsProgram);
  // border
  TextBox->BoxProperties->Border->Width = Style->StandardPixelsToUnits(2);
  TextBox->BoxProperties->Border->Color = clNavy;
  TextBox->BoxProperties->Border->Style = rvbSingle;
  TextBox->BoxProperties->VAlign = tlCenter; // center the image vertically
  // image
  TGraphic* Gr = RVGraphicHandler->LoadFromFile("..\\..\\Snowflake.png");
  if (Gr)
  {
	// center the image horizontally
	TextBox->Document->AddPicture("", Gr, GetCenteredParaStyle(Style));
	// calculating the image size to fit the page
	TRVStyleLength ImageWidth = round(TextBox->BoxProperties->Width * ImageCoeff);
	TRVStyleLength ImageHeight = ImageWidth * Gr->Height / Gr->Width;
	if (ImageHeight > round(TextBox->BoxProperties->Height * ImageCoeff))
	{
	  ImageHeight = round(TextBox->BoxProperties->Height * ImageCoeff);
	  ImageWidth = ImageHeight * Gr->Width / Gr->Height;
	}
	TextBox->Document->SetItemExtraIntProperty(TextBox->Document->ItemCount-1,
	  rvepImageWidth, ImageWidth);
	TextBox->Document->SetItemExtraIntProperty(TextBox->Document->ItemCount-1,
	  rvepImageHeight, ImageHeight);
  }
  // insert
  Header->AddItem("", TextBox);
}
//---------------------------------------------------------------------------
void TfrmMain::GenerateAndSave()
{
  SetDocParameters(SRV);
  GenerateSampleDoc(SRV->RichViewEdit);
  InsertTextBox(SRV);
  SRV->Format();
  SRV->RichViewEdit->SaveRTF("Doc.rtf", false);
  SRV->RichViewEdit->SaveDocX("Doc.docx", false);
}
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::Button1Click(TObject *Sender)
{
  GenerateAndSave();
}
//---------------------------------------------------------------------------

