//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "RVDocParams.hpp"
#include "RVSideNote.hpp"
#include "RVGrHandler.hpp"

#include "MainFrm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;


//---------------------------------------------------------------------------
// Setting A4 page size and margins
void SetDocParameters(TRVDocParameters * DP)
{
  DP->Units = rvuMillimeters;
  DP->PageWidth = 210;
  DP->PageHeight = 297;
  DP->LeftMargin = 30;
  DP->RightMargin = 15;
  DP->TopMargin = 20;
  DP->BottomMargin = 20;
  DP->HeaderY = 10;
  DP->FooterY = 10;
}
//---------------------------------------------------------------------------
void GenerateSampleDoc(TCustomRichView* RV)
{
  const TRVUnicodeString Text =
	"Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
	"Nullam enim turpis, gravida quis justo et, congue aliquam nisi. "
	"Curabitur placerat ex elit. Cras aliquam quam in mi accumsan, "
	"at pellentesque lectus finibus. Donec nisl nulla, scelerisque eget "
	"elit ac, rhoncus mattis eros. Maecenas gravida tincidunt diam, "
	"nec laoreet nulla gravida nec. Class aptent taciti sociosqu ad "
	"litora torquent per conubia nostra, per inceptos himenaeos. "
	"Nulla a metus eleifend nibh aliquam luctus sed vel quam. "
	"Vestibulum lobortis blandit iaculis.";

  for (int i = 0; i <= 500; i++)
	RV->AddNL(Text, 0, 0);
}
//---------------------------------------------------------------------------
void InsertTextBox(TCustomRichView* RV, TRVDocParameters * DP)
{
  const double MarginsCoeff = 0.9;
  const double ImageCoeff = 0.9;
  TRVTextBoxItemInfo* TextBox = new TRVTextBoxItemInfo(RV->RVData);
  // position
  TextBox->BoxPosition->HorizontalPositionKind = rvfpkAbsPosition;
  TextBox->BoxPosition->HorizontalAnchor = rvhanPage;
  TextBox->BoxPosition->HorizontalOffset =
  RV->Style->RVUnitsToUnits(DP->LeftMargin * MarginsCoeff, DP->Units);
  TextBox->BoxPosition->VerticalPositionKind = rvfpkAbsPosition;
  TextBox->BoxPosition->VerticalAnchor = rvvanPage;
  TextBox->BoxPosition->VerticalOffset =
  RV->Style->RVUnitsToUnits(DP->TopMargin  * MarginsCoeff, DP->Units);
  TextBox->BoxPosition->PositionInText = rvpitBelowText;
  // size
  TextBox->BoxProperties->WidthType = rvbwtAbsolute;
  TextBox->BoxProperties->Width =
	RV->Style->RVUnitsToUnits(
	  DP->PageWidth - (DP->LeftMargin + DP->RightMargin)  * MarginsCoeff, DP->Units);
  TextBox->BoxProperties->HeightType = rvbhtAbsolute;
  TextBox->BoxProperties->Height =
	RV->Style->RVUnitsToUnits(
	  DP->PageHeight - (DP->TopMargin + DP->BottomMargin)  * MarginsCoeff, DP->Units);
  // border
  TextBox->BoxProperties->Border->Width = RV->Style->StandardPixelsToUnits(2);
  TextBox->BoxProperties->Border->Color = clNavy;
  TextBox->BoxProperties->Border->Style = rvbSingle;
  TextBox->BoxProperties->VAlign = tlCenter; // center the image vertically
  // image
  TGraphic* Gr = RVGraphicHandler->LoadFromFile("..\\..\\Snowflake.png");
  if (Gr)
  {
	// center the image horizontally (for new TRVStyles, ParaStyles[1] is centered)
	TextBox->Document->AddPicture("", Gr, 1);
	// calculating the image size to fit the page
	TRVStyleLength ImageWidth = round(TextBox->BoxProperties->Width * ImageCoeff);
	TRVStyleLength ImageHeight = ImageWidth * Gr->Height / Gr->Width;
	if (ImageHeight > round(TextBox->BoxProperties->Height * ImageCoeff))
	{
	  ImageHeight = round(TextBox->BoxProperties->Height * ImageCoeff);
	  ImageWidth = ImageHeight * Gr->Width / Gr->Height;
	}
	TextBox->Document->SetItemExtraIntProperty(TextBox->Document->ItemCount-1,
	  rvepImageWidth, ImageWidth);
	TextBox->Document->SetItemExtraIntProperty(TextBox->Document->ItemCount-1,
	  rvepImageHeight, ImageHeight);
  }
  // insert
  RV->AddItem("", TextBox);
}
//---------------------------------------------------------------------------
void TfrmMain::GenerateAndSave()
{
  // creating main RV
  TRichView* RVMain = new TRichView((TComponent*)NULL);
  RVMain->Style = new TRVStyle(RVMain);
  RVMain->Visible = false;
  RVMain->Parent = this;
  TRVRTFOptions RTFOptions = RVMain->RTFOptions << rvrtfSaveHeaderFooter << rvrtfSaveDocParameters;
  RVMain->RTFOptions = RTFOptions;
  SetDocParameters(RVMain->DocParameters);
  GenerateSampleDoc(RVMain);
  // creating header RV
  TRichView* RVHeader = new TRichView((TComponent*)NULL);
  RVHeader->Style = new TRVStyle(RVMain);
  RVHeader->Visible = false;
  RVHeader->Parent = this;
  RVMain->SetHeader(RVHeader->RVData);
  InsertTextBox(RVHeader, RVMain->DocParameters);
  // saving (no need to format before saving, because we do not use tables)
  RVMain->SaveRTF("Doc.rtf", false);
  RVMain->SaveDocX("Doc.docx", false);
  // clean up
  RVMain->Free();
  RVHeader->Free();
}
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::Button1Click(TObject *Sender)
{
  GenerateAndSave();
}
//---------------------------------------------------------------------------

