unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls,
  RVStyle, RichView, RVSideNote, RVFloatingBox, RVFloatingPos, CRVData, RVDocParams,
  RVGrHandler, RVItem, SclRView, RVScroll;

type
  TfrmMain = class(TForm)
    Button1: TButton;
    SRV: TSRichViewEdit;
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Generate;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

{ TfrmMain }

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  Generate;
end;

procedure TfrmMain.Generate;
  {............................................................................}
  // Setting A4 page size and margins
  procedure SetDocParameters;
  var
    PP: TSRVPageProperty;
  begin
    SRV.UnitsProgram := rvuMillimeters;
    PP := SRV.PageProperty;
    PP.PageWidth := 210;
    PP.PageHeight := 297;
    PP.LeftMargin := 30;
    PP.RightMargin := 15;
    PP.TopMargin := 20;
    PP.BottomMargin := 20;
    PP.HeaderY := 10;
    PP.FooterY := 10;
  end;
  {............................................................................}
  procedure GenerateSampleDoc;
  const
    Text = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. ' +
     'Nullam enim turpis, gravida quis justo et, congue aliquam nisi. ' +
     'Curabitur placerat ex elit. Cras aliquam quam in mi accumsan, ' +
     'at pellentesque lectus finibus. Donec nisl nulla, scelerisque eget ' +
     'elit ac, rhoncus mattis eros. Maecenas gravida tincidunt diam, ' +
     'nec laoreet nulla gravida nec. Class aptent taciti sociosqu ad ' +
     'litora torquent per conubia nostra, per inceptos himenaeos. ' +
     'Nulla a metus eleifend nibh aliquam luctus sed vel quam. ' +
     'Vestibulum lobortis blandit iaculis.';
  var
    i: Integer;
  begin
    SRV.RichViewEdit.Clear;
    for i := 0 to 500 do
      SRV.RichViewEdit.AddNL(Text, 0, 0);
  end;
  {............................................................................}
  function GetCenteredParaStyle(ARVStyle: TRVStyle): Integer;
  var
    ParaStyle: TParaInfo;
  begin
    ParaStyle := TParaInfo.Create(nil);
    ParaStyle.Assign(ARVStyle.ParaStyles[0]);
    ParaStyle.Alignment := rvaCenter;
    Result := ARVStyle.FindParaStyle(ParaStyle);
    ParaStyle.Free;
  end;
  {............................................................................}
  procedure InsertTextBox;
  var
    TextBox: TRVTextBoxItemInfo;
    PP: TSRVPageProperty;
    Gr: TGraphic;
    ImageWidth, ImageHeight: TRVStyleLength;
    Header: TCustomRVData;
  const
    MarginsCoeff = 0.9;
    ImageCoeff = 0.9;


  begin
    PP := SRV.PageProperty;
    Header := SRV.SubDocuments[srvhftNormalHeader];
    TextBox := TRVTextBoxItemInfo.Create(Header);
    // position
    TextBox.BoxPosition.HorizontalPositionKind := rvfpkAbsPosition;
    TextBox.BoxPosition.HorizontalAnchor := rvhanPage;
    TextBox.BoxPosition.HorizontalOffset :=
      Header.GetRVStyle.RVUnitsToUnits(PP.LeftMargin * MarginsCoeff, SRV.UnitsProgram);
    TextBox.BoxPosition.VerticalPositionKind := rvfpkAbsPosition;
    TextBox.BoxPosition.VerticalAnchor := rvvanPage;
    TextBox.BoxPosition.VerticalOffset :=
      Header.GetRVStyle.RVUnitsToUnits(PP.TopMargin  * MarginsCoeff, SRV.UnitsProgram);
    TextBox.BoxPosition.PositionInText := rvpitBelowText;
    // size
    TextBox.BoxProperties.WidthType := rvbwtAbsolute;
    TextBox.BoxProperties.Width :=
      Header.GetRVStyle.RVUnitsToUnits(
        PP.PageWidth - (PP.LeftMargin + PP.RightMargin)  * MarginsCoeff, SRV.UnitsProgram);
    TextBox.BoxProperties.HeightType := rvbhtAbsolute;
    TextBox.BoxProperties.Height :=
      Header.GetRVStyle.RVUnitsToUnits(
        PP.PageHeight - (PP.TopMargin + PP.BottomMargin)  * MarginsCoeff, SRV.UnitsProgram);
    // border
    TextBox.BoxProperties.Border.Width := Header.GetRVStyle.StandardPixelsToUnits(2);
    TextBox.BoxProperties.Border.Color := clNavy;
    TextBox.BoxProperties.Border.Style := rvbSingle;
    TextBox.BoxProperties.VAlign := tlCenter; // center the image vertically
    // image
    Gr := RVGraphicHandler.LoadFromFile('..\..\Snowflake.png');
    if Gr <> nil then
    begin
      TextBox.Document.AddPicture('', Gr, GetCenteredParaStyle(Header.GetRVStyle)); // center the image horizontally
      // calculating the image size to fit the page
      ImageWidth := Round(TextBox.BoxProperties.Width * ImageCoeff);
      ImageHeight := ImageWidth * Gr.Height div Gr.Width;
      if ImageHeight > Round(TextBox.BoxProperties.Height * ImageCoeff) then
      begin
        ImageHeight := Round(TextBox.BoxProperties.Height * ImageCoeff);
        ImageWidth := ImageHeight * Gr.Width div Gr.Height;
      end;
      TextBox.Document.SetItemExtraIntProperty(TextBox.Document.ItemCount-1,
        rvepImageWidth, ImageWidth);
      TextBox.Document.SetItemExtraIntProperty(TextBox.Document.ItemCount-1,
        rvepImageHeight, ImageHeight);
    end;
    // insert
    Header.AddItem('', TextBox);
  end;
  {............................................................................}
begin
  SetDocParameters;
  GenerateSampleDoc;
  InsertTextBox;
  SRV.Format;
  SRV.RichViewEdit.SaveRTF('Doc.rtf', False);
  SRV.RichViewEdit.SaveDocX('Doc.docx', False);
end;


end.
