unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls,
  RVStyle, RichView, RVSideNote, RVFloatingBox, RVFloatingPos, CRVData, RVDocParams,
  RVGrHandler, RVItem;

type
  TfrmMain = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Generate;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

{ TfrmMain }

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  Generate;
end;

procedure TfrmMain.Generate;
var
  RVMain, RVHeader: TRichView;
  {............................................................................}
  // Setting A4 page size and margins
  procedure SetDocParameters;
  var
    DP: TRVDocParameters;
  begin
    DP := RVMain.DocParameters;
    DP.Units := rvuMillimeters;
    DP.PageWidth := 210;
    DP.PageHeight := 297;
    DP.LeftMargin := 30;
    DP.RightMargin := 15;
    DP.TopMargin := 20;
    DP.BottomMargin := 20;
    DP.HeaderY := 10;
    DP.FooterY := 10;
  end;
  {............................................................................}
  procedure GenerateSampleDoc;
  const
    Text = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. ' +
     'Nullam enim turpis, gravida quis justo et, congue aliquam nisi. ' +
     'Curabitur placerat ex elit. Cras aliquam quam in mi accumsan, ' +
     'at pellentesque lectus finibus. Donec nisl nulla, scelerisque eget ' +
     'elit ac, rhoncus mattis eros. Maecenas gravida tincidunt diam, ' +
     'nec laoreet nulla gravida nec. Class aptent taciti sociosqu ad ' +
     'litora torquent per conubia nostra, per inceptos himenaeos. ' +
     'Nulla a metus eleifend nibh aliquam luctus sed vel quam. ' +
     'Vestibulum lobortis blandit iaculis.';
  var
    i: Integer;
  begin
    for i := 0 to 500 do
      RVMain.AddNL(Text, 0, 0);
  end;
  {............................................................................}
  procedure InsertTextBox;
  var
    TextBox: TRVTextBoxItemInfo;
    DP: TRVDocParameters;
    Gr: TGraphic;
    ImageWidth, ImageHeight: TRVStyleLength;
  const
    MarginsCoeff = 0.9;
    ImageCoeff = 0.9;
  begin
    DP := RVMain.DocParameters;
    TextBox := TRVTextBoxItemInfo.Create(RVHeader.RVData);
    // position
    TextBox.BoxPosition.HorizontalPositionKind := rvfpkAbsPosition;
    TextBox.BoxPosition.HorizontalAnchor := rvhanPage;
    TextBox.BoxPosition.HorizontalOffset :=
      RVHeader.Style.RVUnitsToUnits(DP.LeftMargin * MarginsCoeff, DP.Units);
    TextBox.BoxPosition.VerticalPositionKind := rvfpkAbsPosition;
    TextBox.BoxPosition.VerticalAnchor := rvvanPage;
    TextBox.BoxPosition.VerticalOffset :=
      RVHeader.Style.RVUnitsToUnits(DP.TopMargin  * MarginsCoeff, DP.Units);
    TextBox.BoxPosition.PositionInText := rvpitBelowText;
    // size
    TextBox.BoxProperties.WidthType := rvbwtAbsolute;
    TextBox.BoxProperties.Width :=
      RVHeader.Style.RVUnitsToUnits(
        DP.PageWidth - (DP.LeftMargin + DP.RightMargin)  * MarginsCoeff, DP.Units);
    TextBox.BoxProperties.HeightType := rvbhtAbsolute;
    TextBox.BoxProperties.Height :=
      RVHeader.Style.RVUnitsToUnits(
        DP.PageHeight - (DP.TopMargin + DP.BottomMargin)  * MarginsCoeff, DP.Units);
    // border
    TextBox.BoxProperties.Border.Width := RVHeader.Style.StandardPixelsToUnits(2);
    TextBox.BoxProperties.Border.Color := clNavy;
    TextBox.BoxProperties.Border.Style := rvbSingle;
    TextBox.BoxProperties.VAlign := tlCenter; // center the image vertically
    // image
    Gr := RVGraphicHandler.LoadFromFile('..\..\Snowflake.png');
    if Gr <> nil then
    begin
      // center the image horizontally (for new TRVStyles, ParaStyles[1] is centered)
      TextBox.Document.AddPicture('', Gr, 1);
      // calculating the image size to fit the page
      ImageWidth := Round(TextBox.BoxProperties.Width * ImageCoeff);
      ImageHeight := ImageWidth * Gr.Height div Gr.Width;
      if ImageHeight > Round(TextBox.BoxProperties.Height * ImageCoeff) then
      begin
        ImageHeight := Round(TextBox.BoxProperties.Height * ImageCoeff);
        ImageWidth := ImageHeight * Gr.Width div Gr.Height;
      end;
      TextBox.Document.SetItemExtraIntProperty(TextBox.Document.ItemCount-1,
        rvepImageWidth, ImageWidth);
      TextBox.Document.SetItemExtraIntProperty(TextBox.Document.ItemCount-1,
        rvepImageHeight, ImageHeight);
    end;
    // insert
    RVHeader.AddItem('', TextBox);
  end;
  {............................................................................}
begin
  // creating main RV
  RVMain := TRichView.Create(nil);
  RVMain.Style := TRVStyle.Create(RVMain);
  RVMain.Visible := False;
  RVMain.Parent := Self;
  RVMain.RTFOptions := RVMain.RTFOptions +
    [rvrtfSaveHeaderFooter, rvrtfSaveDocParameters];
  SetDocParameters;
  GenerateSampleDoc;
  // creating header RV
  RVHeader := TRichView.Create(nil);
  RVHeader.Style := TRVStyle.Create(RVMain);
  RVHeader.Visible := False;
  RVHeader.Parent := Self;
  RVMain.SetHeader(RVHeader.RVData);
  InsertTextBox;
  // saving (no need to format before saving, because we do not use tables)
  RVMain.SaveRTF('Doc.rtf', False);
  RVMain.SaveDocX('Doc.docx', False);
  // clean up
  RVMain.Free;
  RVHeader.Free;
end;

end.
